﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using HLWebOKBQueryPlugin.Models;
using HLWebOKBQueryPlugin.Helpers;
using HLWebOKBQueryPlugin.OKBQueryService;
using System.Web.Routing;

namespace HLWebOKBQueryPlugin.Controllers
{
    /// <summary>
    /// Controller for the filter subpage.
    /// </summary>
    public class FilterController : Controller
    {
        /// <summary>
        /// Get run ids from service.
        /// </summary>
        /// <returns></returns>
        public ActionResult GetRuns()
        {
            FilterModel qm = new FilterModel();
            qm.Content = (CombinedFilter)Session["Content"];

            QueryServiceClient client = Query.GetClientFactory("okbtester", "okbtester");
           long[] ids =  client.GetRunIds(qm.Content);

            Response.Write("Found <" + ids.Count() + "> Runs.");

            return View("Index", qm); // name of usercontrol
        }

        public ActionResult Filters(CombinedFilter f)
        {
            FilterModel qm = new FilterModel(f);
            return PartialView("Filters", qm); // name of usercontrol
        }


        /// <summary>
        /// Add a new Filter, based on the selected combobox
        /// value. Posted values will also be set to the 
        /// "right" filter(s).
        /// </summary>
        /// <param name="collection"></param>
        /// <returns></returns>
        public ActionResult AddFilterAnd(FormCollection collection)
        {
            CombinedFilter content = (CombinedFilter)Session["Content"];
            FilterModel fm = new FilterModel(content);

            

            String idKey = null;
            String parentIdKey = null;

            foreach (string key in collection.AllKeys)
            {
                if (key.StartsWith("filtersCombobox"))
                {
                    idKey = key;
                }
                if (key.StartsWith("parentId"))
                {
                    parentIdKey = key;
                }
            }

            Guid parentId = new Guid(collection.Get(parentIdKey));
            string filterTypeName = collection.Get(idKey);

            // get selected filter (combobox)
            Filter filter = fm.Filters.Where(x => x.FilterTypeName.Equals(filterTypeName)).FirstOrDefault();



            // find the partent filterr
            List<Filter> orFilters = content.Filters.ToList<Filter>(); ;
            CombinedFilter parent = (CombinedFilter)orFilters.Where(e => e.Id.Equals(parentId)).FirstOrDefault();

            // remove
            orFilters.Remove(parent);

            List<Filter> filterList = parent.Filters.ToList<Filter>();




            // Iterate "posted" values and build up a dictionary for
            // each filter (id).
            // <id>     =>      <key> => <value>
            //          =>      <key> => <value>
            Dictionary<Guid, Dictionary<string, string>> postValues = new Dictionary<Guid, Dictionary<string, string>>();

            foreach (string key in collection.AllKeys)
            {

                if (!key.StartsWith("filtersCombobox") && !key.StartsWith("parentId"))
                {
                    int idx = key.IndexOf(".");
                    string id = key.Substring(0, idx);
                    string type = key.Substring(idx + 1);

                    Guid fid = new Guid(id);

                    if (postValues.ContainsKey(fid))
                    {
                        Dictionary<string, string> d = postValues[fid];
                        d.Add(type, collection.Get(key));
                    }
                    else
                    {
                        Dictionary<string, string> d = new Dictionary<string, string>();
                        d.Add(type, collection.Get(key));
                        postValues.Add(fid, d);
                    }
                }
            }


            // Set values for the existing Filters
            foreach (Filter f in filterList)
            {

                Dictionary<string, string> dict = postValues.Where(x => x.Key.Equals(f.Id)).FirstOrDefault().Value;
                if (dict != null)
                {
                    foreach (KeyValuePair<string, string> kvp in dict)
                    {
                        if ("StringComparisonFilter".Equals(f.GetType().Name))
                        {
                            if (kvp.Key.Equals("valueTextbox"))
                            {
                                ((StringComparisonFilter)f).Value = kvp.Value;
                            }

                        }
                        else
                        {
                            // To this for all filters
                        }
                    }
                }

            }


            // Add the new filter
            filterList.Add(filter);
            parent.Filters = filterList.ToArray<Filter>();

            // Add the current filter (AND)
            orFilters.Add(parent);

            content.Filters = orFilters.ToArray<Filter>();

            // Set Content
            Session["Content"] = content;



            return View("Index", fm);
        }


        /// <summary>
        /// Add a new Filter "Box" for OR connection.
        /// </summary>
        /// <returns></returns>
        public ActionResult AddFilterOr()
        {


            CombinedFilter content = (CombinedFilter)Session["Content"];
            FilterModel fm = new FilterModel(content);

            List<Filter> orFilters = content.Filters.ToList<Filter>(); ;

            CombinedFilter andFilter = fm.Filters.OfType<CombinedFilter>().Where(x => x.Operation == BooleanOperation.And).FirstOrDefault();
            orFilters.Add(andFilter);
            content.Filters = orFilters.ToArray<Filter>();


            Session["Content"] = content;

            return View("Index", fm);
        }




        /// <summary>
        /// This action will be called when rendering
        /// the index.aspx. This will hapen the first
        /// time, so the available Filters in the Model
        /// will be set.
        /// </summary>
        /// <returns></returns>
        public ActionResult Index()
        {
           
            // Set submenu
            Session["SelectedSubMenu"] = "Filter";
            FilterModel fm = new FilterModel();


            // Init session variable for selected filters
            if (Session["Content"] == null)
            {
                // Add first Filter 
                CombinedFilter orFilter = fm.Filters.OfType<CombinedFilter>().Where(x => x.Operation == BooleanOperation.Or).FirstOrDefault();
                CombinedFilter andFilter = fm.Filters.OfType<CombinedFilter>().Where(x => x.Operation == BooleanOperation.And).FirstOrDefault();
                orFilter.Filters = new Filter[] { andFilter }; // 1st filter

                Session["Content"] = orFilter;
            }

            // Set model
            CombinedFilter f = (CombinedFilter)Session["Content"];
            fm.Content = f;
            
           
            return View(fm);
        }


    }
}
